Documentation
¶
Overview ¶
Package test provides common functions for testing.
Index ¶
Constants ¶
This section is empty.
Variables ¶
var UpdateOut = flag.Bool("update", false, "Update the example files in test/data and test/data/out")
UpdateOut is a flag that can be set to update example files
Functions ¶
func LoadEnvelope ¶
LoadEnvelope loads a test file from the test/data folder as a GOBL envelope and will rebuild it if necessary to ensure any changes are accounted for.
func LoadInvoice ¶
LoadInvoice will load the GOBL Envelope and extract the invoice, returning both the Envelope and Invoice objects.
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.